import os
import sys
from pathlib import Path

from datetime import datetime
import click
# import matplotlib
import numpy as np
from matplotlib import container
from matplotlib.backends.backend_qt5agg import FigureCanvas
# from matplotlib.backends.qt_compat import QtCore, QtGui, QtWidgets
from matplotlib.figure import Figure
from PyQt5 import QtCore, QtGui, QtWidgets

# matplotlib.use('Qt5Agg')


from input import read
from navigation_toolbar import NavigationToolbar

__title__ = 'Aprespy'
__version__ = '2021.4.1'


os.environ['QT_MAC_WANTS_LAYER'] = '1'


class Aprespy(QtWidgets.QWidget):
    """Summary."""

    XX = 0
    XY = 1
    YX = 2
    YY = 3
    TX = 0
    TY = 1
    MARK = ['o', 'o', 'x', 'x']
    COLOR = ['#009900', '#0000B3', '#B30000', '#CECE00']
    # COLOR = ['#5AB300', '#00B3B3', '#B30000', '#5A00B3']
    # COLOR = ['g', 'b', 'r', 'y']
    FACE_COLOR = '#EDEDED'

    def __init__(self, file_name, data):
        super(Aprespy, self).__init__()
        self.title = file_name
        self.data = data
        self.cur_rot = data.data[data.rotations.index(data.theta)]
        self.rotations = data.rotations
        self.theta = data.theta
        self.standard_axis = False
        self.error_guide = False
        self.diagonals = False
        self.error_guide_plots = []

        # Get the path to the img directory, need when bundling with pyinstaller
        if getattr(sys, 'frozen', False):
            # we are running in a bundle
            img_pth = os.path.join(sys._MEIPASS, 'img')
        elif os.path.exists('img'):
            img_pth = 'img'
        else:
            img_pth = os.path.join('aprespy', 'img')

        self._init_ui(img_pth)
        self._init_layout()
        self._plot()
        self._legend()
        self._tipper_legend()

        # Initialize standard y axis
        self.standard_yaxis.click()

    def _init_ui(self, img_pth: str):
        self.setWindowTitle(f'{__title__} v{__version__}')
        self.setMinimumSize(800, 700)
        self.layout = QtWidgets.QVBoxLayout(self)
        self.setContentsMargins(0, 0, 0, 0)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.canvas = FigureCanvas(Figure(facecolor=self.FACE_COLOR))
        self.toolbar = self._init_toolbar(img_pth)
        self.layout.addWidget(self.toolbar)
        self.layout.addWidget(self.canvas)

    def _init_toolbar(self, img_pth: str):
        # Initialize buttons
        self.standard_yaxis = self._init_standard_yaxis(img_pth)
        self.diagonal = self._init_diagonal(img_pth)
        self.error_guide_button = self._init_error_guide(img_pth)
        self.rotation_ang = self._init_rotation_ang_label()
        self.rotation_slider = self._init_rotation_slider(self.rotations)

        # Add buttons to toolbar
        toolbar = NavigationToolbar(self.canvas, self)
        toolbar.addWidget(self.standard_yaxis)
        toolbar.addWidget(self.diagonal)
        toolbar.addWidget(self.error_guide_button)
        toolbar.addSeparator()
        toolbar.addWidget(self.rotation_ang)
        toolbar.addWidget(self.rotation_slider)

        # Connect the widgets
        self.standard_yaxis.toggled.connect(self._standard_yaxis_toggled)
        self.diagonal.toggled.connect(self._diagonal_toggled)
        self.error_guide_button.toggled.connect(self._error_guide_toggled)
        # self._rotation_slider.valueChanged.connect(self._slider_changed)
        self.rotation_slider.valueChanged.connect(self._rotation_angle_changed)
        # self._rotation_slider.sliderReleased.connect(self._rotation_angle_changed)

        return toolbar

    def _init_standard_yaxis(self, img_pth: str):
        # add default axis
        default_yaxis = QtWidgets.QToolButton()
        default_yaxis.setToolTip('Set standard rho yaxis')
        default_yaxis.setIcon(QtGui.QIcon(os.path.join(img_pth,
                                                       'default_axis.png')))
        default_yaxis.setCheckable(True)

        return default_yaxis

    def _init_rotation_ang_label(self):
        d = u"\u00b0"
        label = QtWidgets.QLabel(self)
        label.setText(f'{self.theta}{d}')
        label.setFixedWidth(45)
        label.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        return label

    def _init_diagonal(self, img_pth: str):
        # add default axis
        diagonal = QtWidgets.QToolButton()
        diagonal.setToolTip('Toggle diagonal components')
        diagonal.setIcon(QtGui.QIcon(os.path.join(img_pth,
                                                  'diagonal.png')))
        diagonal.setCheckable(True)

        return diagonal

    def _init_error_guide(self, img_pth: str):
        # add default axis
        error_guide = QtWidgets.QToolButton()
        error_guide.setToolTip('Display error bar guide')
        error_guide.setIcon(QtGui.QIcon(os.path.join(img_pth,
                                                     'guide.png')))

        error_guide.setCheckable(True)

        return error_guide

    def _init_rotation_slider(self, rot: list = []):
        sl = QtWidgets.QSlider(QtCore.Qt.Horizontal)
        sl.setFocusPolicy(QtCore.Qt.StrongFocus)
        sl.TickPosition(QtWidgets.QSlider.TicksBelow)
        sl.setRange(0, len(rot) - 1)
        # TODO: set initial slider value
        for i, ang in enumerate(self.rotations):
            # if d.measured_coor:
            if ang == self.theta:
                sl.setValue(i)
        return sl

    def _init_layout(self):
        d = u"\u00B7"
        gs = self.canvas.figure.add_gridspec(3, 5)
        self.axr = self.canvas.figure.add_subplot(gs[0:2, 0:3])
        self.axr.set_title(f'{self.title}', fontweight='bold')
        self.axr.set_ylabel('Resistivity (Ω m)', fontweight='bold')
        self.axr.tick_params(labelbottom=False)
        self._set_rotation_angle_labels()

        self.axr.set_xscale('log')
        self.axr.set_yscale('log')

        self.axp = self.canvas.figure.add_subplot(gs[2, 0:3], sharex=self.axr)
        self.axr.grid(True, which='major', linestyle='solid')
        self.axr.grid(True, which='minor', linestyle='dotted')
        self.axp.set_ylabel('Phase (°)', fontweight='bold')
        self.axp.set_xlabel('Period (s)', fontweight='bold')

        self.axp.yaxis.set_ticks(np.arange(-90, 91, 15))
        self.axp.set_ylim(ymin=-5, ymax=95)

        self.axp.grid(True, which='major', linestyle='solid')
        self.axp.grid(True, which='minor', linestyle='dotted')

        # TODO: Adjust base on the tipper values
        tlim = [-0.25, 0.25]
        ttik = [tlim[0] / 2, 0, tlim[1] / 2]
        self.atx = self.canvas.figure.add_subplot(gs[:, 3])
        self.atx.set_yscale('log')
        self.atx.set_xlim(xmin=tlim[0], xmax=tlim[1])
        self.atx.xaxis.set_ticks(ttik)
        self.atx.set_xticklabels(ttik, fontsize=8)
        self.atx.set_ylim(ymin=40_000, ymax=4)
        self.atx.tick_params(labelleft=False)
        self.atx.set_xlabel('|Tzx|', fontweight='bold')
        self.atx.grid(True, which='major', linestyle='dashed')

        self.aty = self.canvas.figure.add_subplot(gs[:, 4], sharey=self.atx)
        self.aty.set_xlabel('|Tzy|', fontweight='bold')
        self.aty.grid(True, which='major', linestyle='dashed')
        self.aty.set_xlim(xmin=tlim[0], xmax=tlim[1])
        self.aty.xaxis.set_ticks(ttik)
        self.aty.set_xticklabels(ttik, fontsize=8)
        self.aty.yaxis.tick_right()
        self.aty.yaxis.set_label_position('right')
        self.aty.set_ylabel('Period (s)', fontweight='bold')

    def _plot(self):
        if self.error_guide:
            self._plot_error_guide(self.XY)
        if self.diagonals:
            self.rxx, self.pxx = self._plot_curves('Zxx', self.XX)
            self.ryy, self.pyy = self._plot_curves('Zyy', self.YY)
        self.rxy, self.pxy = self._plot_curves('Zxy', self.XY)
        self.ryx, self.pyx = self._plot_curves('Zyx', self.YX)
        self.zxr, self.zxi = self._plot_tipper(self.atx, self.TX)
        self.zyr, self.zyi = self._plot_tipper(self.aty, self.TY)

    def _plot_curves(self, label, position):
        rho = self._plot_line(self.axr,
                              self.data.periods,
                              self.cur_rot.rho[:, position],
                              None,
                              self.cur_rot.rho_se[:, position],
                              self.MARK[position],
                              self.COLOR[position],
                              label)
        phi = self._plot_line(self.axp,
                              self.data.periods,
                              self.cur_rot.phi[:, position],
                              None,
                              self.cur_rot.phi_se[:, position],
                              self.MARK[position],
                              self.COLOR[position],
                              label)

        return rho, phi

    def _plot_tipper(self, plot, position):
        real = self._plot_line(plot,
                               self.cur_rot.tz[:, position].real,
                               self.data.periods,
                               self.cur_rot.tz_se[:, position].real,
                               None,
                               self.MARK[self.XY],
                               'm',
                               'Re(Tz)')
        imag = self._plot_line(plot,
                               self.cur_rot.tz[:, position].imag,
                               self.data.periods,
                               self.cur_rot.tz_se[:, position].real,
                               None,
                               self.MARK[self.YX],
                               'k',
                               'Im(Tz)')

        if self.cur_rot.tz.shape[0] > 23:
            trmid = self.cur_rot.tz[14:24, position].real
            trmax = np.abs(max(trmid))
            trmin = np.abs(min(trmid))
            trmm = max([trmax, trmin])
            if trmm < 0.25:
                tbounds = [-0.25, 0.25]
                ttik = [tbounds[0] / 2, 0, tbounds[1] / 2]
            elif trmm < 0.5 and trmm > 0.25:
                tbounds = [-0.5, 0.5]
                ttik = [tbounds[0] / 2, 0, tbounds[1] / 2]
            elif trmm < 0.75 and trmm > 0.5:
                tbounds = [-0.75, 0.75]
                ttik = [tbounds[0] / 2, 0, tbounds[1] / 2]
            else:
                tbounds = [-1, 1]
                ttik = [tbounds[0] / 2, 0, tbounds[1] / 2]

            plot.set_xlim(xmin=tbounds[0], xmax=tbounds[1])
            plot.xaxis.set_ticks(ttik)
            plot.set_xticklabels(ttik, fontsize=8)

        return real, imag

    def _plot_error_guide(self, position):
        self.error_guide_plots = []
        start = 4
        end = 29
        periods = self.data.periods[start:end]
        zxy = self.cur_rot.rho[start:end, self.XY]
        zyx = self.cur_rot.rho[start:end, self.YX]
        error = np.zeros(zyx.shape)
        zxy_guide = [np.zeros(zyx.shape), np.zeros(zyx.shape)]
        zyx_guide = [np.zeros(zyx.shape), np.zeros(zyx.shape)]
        for i in range(error.shape[0]):
            zxy_guide[0][i] = zxy[i] - zxy[i] * 0.05
            zxy_guide[1][i] = zxy[i] + zxy[i] * 0.05
            zyx_guide[0][i] = zyx[i] - zyx[i] * 0.05
            zyx_guide[1][i] = zyx[i] + zyx[i] * 0.05

        zxy_fill = self.axr.fill_between(periods, zxy_guide[0], zxy_guide[1], color='k', alpha=0.05)
        zyx_fill = self.axr.fill_between(periods, zyx_guide[0], zyx_guide[1], color='k', alpha=0.05)
        zxy_low, = self.axr.plot(periods, zxy_guide[0], color='k', linewidth=0.25)
        zxy_high, = self.axr.plot(periods, zxy_guide[1], color='k', linewidth=0.25)
        zyx_low, = self.axr.plot(periods, zyx_guide[0], color='k', linewidth=0.25)
        zyx_high, = self.axr.plot(periods, zyx_guide[1], color='k', linewidth=0.25)
        self.error_guide_plots.append(zxy_fill)
        self.error_guide_plots.append(zyx_fill)
        self.error_guide_plots.append(zxy_low)
        self.error_guide_plots.append(zxy_high)
        self.error_guide_plots.append(zyx_low)
        self.error_guide_plots.append(zyx_high)

        pxy = self.cur_rot.phi[start:end, self.XY]
        pyx = self.cur_rot.phi[start:end, self.YX]
        pxy_fill = self.axp.fill_between(periods, pxy - 2, pxy + 2, color='k', alpha=0.05)
        pyx_fill = self.axp.fill_between(periods, pyx - 2, pyx + 2, color='k', alpha=0.05)
        pxy_low, = self.axp.plot(periods, pxy - 2, color='k', linewidth=0.25)
        pxy_high, = self.axp.plot(periods, pxy + 2, color='k', linewidth=0.25)
        pyx_low, = self.axp.plot(periods, pyx - 2, color='k', linewidth=0.25)
        pyx_high, = self.axp.plot(periods, pyx + 2, color='k', linewidth=0.25)
        self.error_guide_plots.append(pxy_fill)
        self.error_guide_plots.append(pyx_fill)
        self.error_guide_plots.append(pxy_low)
        self.error_guide_plots.append(pxy_high)
        self.error_guide_plots.append(pyx_low)
        self.error_guide_plots.append(pyx_high)

    def _plot_line(self, plot, x: np.array, y: np.array, x_err: np.array, y_err: np.array,
                   marker: str, color: str, label: str):
        line = plot.errorbar(x, y, xerr=x_err, yerr=y_err, color=color,
                             marker=marker, linestyle='none', mfc='none',
                             capsize=3, label=label)
        return line

    def _legend(self):
        # Define the initial handle labels and order
        label_set = ['Zxy', 'Zyx']
        if self.diagonals:
            label_set = ['Zxx'] + label_set + ['Zyy']

        # Get line handles and labels from the graph
        handles, labels = self.axr.get_legend_handles_labels()
        # Remove the duplicate handles/labels and reorder
        handles = [handles[labels.index(i)] for i in label_set]
        labels = [labels[labels.index(i)] for i in label_set]
        # Remove the error bars from the legend symbols
        handles = [h[0] if isinstance(h, container.ErrorbarContainer) else h for h in handles]

        legend = self.axr.legend(handles, labels, loc='upper left')
        legend.set_draggable(state=True)

    def _tipper_legend(self):
        plot = self.aty
        label_set = ['Re(Tz)', 'Im(Tz)']

        # Get line handles and labels from the graph
        handles, labels = plot.get_legend_handles_labels()
        # Remove the duplicate handles/labels and reorder
        handles = [handles[labels.index(i)] for i in label_set]
        labels = [labels[labels.index(i)] for i in label_set]
        # Remove the error bars from the legend symbols
        handles = [h[0] if isinstance(h, container.ErrorbarContainer) else h for h in handles]

        legend = plot.legend(handles, labels, loc='upper left')
        legend.set_draggable(state=True)

    def _set_rotation_angle_labels(self):
        d = u"\u00b0"
        self.rotation_ang.setText(f'{self.theta}{d}')
        self.axr.set_xlabel(f'Geodetic Rotation: {self.theta}{d}')

    def _standard_yaxis_toggled(self, checked: bool):
        if checked:
            self.standard_axis = True
            self._set_rho_y_axis()
        else:
            self.standard_axis = False
            self._reload_graphs()

    def _diagonal_toggled(self, checked: bool):
        if checked:
            self.diagonals = True
            self._reload_graphs()
        else:
            self.diagonals = False
            self._reload_graphs()

    def _error_guide_toggled(self, checked: bool):
        if checked:
            self.error_guide = True
            self._reload_graphs()
        else:
            self.error_guide = False
            self._reload_graphs()

    def _set_rho_y_axis(self):
        self.standard_axis = True
        median = int(np.median(self.cur_rot.rho[:, self.XY]) +
                     np.median(self.cur_rot.rho[:, self.YX]))
        median = int(median / 2)
        self.axr.set_ylim(ymin=median / 100, ymax=median * 100)
        self.canvas.draw()

    def _slider_changed(self):
        self.theta = self.rotations[self._rotation_slider.value()]
        self._set_rotation_angle_labels()

    def _rotation_angle_changed(self):
        self.theta = self.rotations[self.rotation_slider.value()]
        self.cur_rot = self.data.data[self.rotations.index(self.theta)]
        self._set_rotation_angle_labels()
        self._reload_graphs()

    def _reload_graphs(self):
        try:
            for line in self.error_guide_plots:
                line.remove()
        except (AttributeError, ValueError):
            pass
        try:
            self.rxx.remove()
            self.ryy.remove()
            self.pxx.remove()
            self.pyy.remove()
        except (AttributeError, ValueError):
            pass
        self.rxy.remove()
        self.ryx.remove()
        self.pxy.remove()
        self.pyx.remove()
        self.zxr.remove()
        self.zxi.remove()
        self.zyr.remove()
        self.zyi.remove()
        self._plot()
        self._legend()
        if self.standard_axis:
            self._set_rho_y_axis()
        else:
            self.axr.autoscale()
            self.canvas.draw()

        self.canvas.flush_events()

    def save(self, path: str = None):
        file_dir = ''
        file_name = Path(self.title).stem + '.png'

        if path is not None:
            try:
                path = Path(path)
                path = path.parent.parent
                file_dir = os.path.join(path, 'plot')
                if not os.path.exists(file_dir):
                    os.makedirs(file_dir)
            except OSError:
                file_dir = ''

        file_path = os.path.join(file_dir, file_name)

        print(f'Saving image: {file_path}')

        self.canvas.figure.set_size_inches(18, 12)
        self.canvas.figure.savefig(file_path, dpi=100)


def file_dialog(prev_dir: str = str(Path.home())):
    file_path, _ = QtWidgets.QFileDialog.getOpenFileName(None,
                                                         'Open File dialog',
                                                         prev_dir,
                                                         filter='((*.zss *.zrr)'
                                                         )
    if file_path != '':
        return file_path


def plot(file_path: str = None, save: bool = False, quit: bool = False):
    app = QtWidgets.QApplication(sys.argv)
    if file_path is None:
        file_path = file_dialog()
        if file_path is None:
            return

    if not os.path.exists(file_path):
        print(f'File not found: {file_path}')
        sys.exit()

    file_name = Path(file_path).name
    print(f'Reading file: {file_path}')
    data = read(file_path)

    window = Aprespy(file_name, data)
    if save is True:
        window.save(file_path)

    if quit is True:
        sys.exit()

    window.show()
    sys.exit(app.exec_())

def wait_for_file(path: str = None):
    SECONDS = 20
    if str is None:
        return None

    start = datetime.now()
    print('Waiting for file...')
    while True:
        if os.path.exists(path):
            break
        elif (datetime.now() - start).total_seconds() > 10:
            print(f'ERROR: {SECONDS} seconds have passed, Exiting')
            break


@click.command()
@click.option('-p', '--path', help='zss/zrr file path')
@click.option('-s', '--save', is_flag=True, help='save png file')
@click.option('-ng', '--no_gui', is_flag=True, help='exit without displaying the gui')
@click.option('-ss', '--ss', is_flag=True, help='flag to check if it is running in matlab SS mode')
def main(path: str = None, save: bool = False, no_gui: bool = False, ss: bool = False):
    if ss is True and path is not None:
        wait_for_file(path)
    plot(path, save, no_gui)


if __name__ == "__main__":
    main()



import sys
from collections import namedtuple
from typing import List, Tuple

import numpy as np


class Data:

    def __init__(self):
        self.theta = 0
        self.rotations = [i for i in range(0, 91, 5)]
        self.periods = []
        self.data = []

    def append_then_sort(self, value):
        self.rotations.append(value)
        self.rotations = list(set(self.rotations))
        self.rotations = sorted(self.rotations)


def read(path: str) -> list:
    """Read in z file information.

    Read in the z file information and output a list of rotated rho, phi,
    and error parameters.

    Args:
        path: Z file path.

    Returns:
        data: List of Rotation class information.

    """
    data = Data()

    # Read in z file
    z_params = z_in(path)

    # Get the electrode orientations
    (xy, yx, hz, xy_pairs, dipole_setup) = ecomp(z_params['nch'], z_params['chid'])

    # Get the site declination
    data.theta = z_params['stdec'].dec
    data.periods = z_params['periods'][0]

    # Add in the measured angle
    data.append_then_sort(data.theta)

    # Assemble arrays
    (z2x2, sigs, sige) = z_to_imp(z_params['z'], z_params['sig_s'],
                                  z_params['sig_e'], z_params['nche'],
                                  xy_pairs, z_params['orient'])

    for r in data.rotations:
        tz, tz_se = z_to_tz(z_params['z'], z_params['sig_s'], z_params['sig_e'], len(z_params['periods'][0]), r - data.theta)
        # Apply rotations oriented to True North
        if z_params['dec_format'] is True:
            (z2x2r, sigsr, siger) = rot_z(z2x2, sigs, sige, r - data.theta)
        else:
            (z2x2r, sigsr, siger) = rot_z(z2x2, sigs, sige, r)
        # Convert to apparent resistivity and phase
        data.data.append(imp_ap(z2x2r, sigsr, siger, z_params['periods'], tz, tz_se))

    return data


def z_in(path: str):
    """Read information in Z file and parse said information into a dict.

    Args:
        flepth: Z file path.

    Returns:
        z_params: Dictionary of z file information.

    """
    z_params = dict()

    # Read lines from file
    with open(path) as f:
        lines = f.readlines()

    # Parse *.z* file information
    station = lines[3]
    cordec = lines[4]
    nchans = lines[5]
    nch = int(nchans.split()[3])
    metdat = lines[7: 7 + nch]
    tfdata = lines[8 + nch:]

    # declare variables
    nche = nch - 2
    nbt = int(nchans.split()[7])
    z = np.zeros((2, nche * nbt), dtype=np.complex)
    sig_s = np.zeros((2, 2 * nbt), dtype=np.complex)
    sig_e = np.zeros((nche, nche * nbt), dtype=np.complex)
    periods = np.zeros((1, nbt), dtype=np.float64)
    ndf = np.zeros((nbt, 1), dtype=np.int64)
    Coordec = namedtuple('Coordec', 'lat long dec')
    orient = np.zeros((2, nch), dtype=np.float64)
    chid = []
    csta = []
    station = ''

    # get the station id
    station = station.split(":").pop()
    # Parse orientation, tilt, channel id, and channel station name
    for i, chan in enumerate(metdat):
        fields = chan.split()
        fields.pop(0)
        # Handle negative orientation
        for j in range(2):
            o = float(fields.pop(0))
            if o < 0:
                orient[j][i] = 180 + o
            else:
                orient[j][i] = o
        csta.append(fields.pop(0))
        chid.append(fields.pop(0))
    # get coordinates and declination
    fields = cordec.split()
    if fields[4] == '0.00':
        dec = orient[0][0]
        dec_format = False
    else:
        dec = float(fields[4])
        dec_format = True
    stdec = Coordec(lat=float(fields[1]),
                    long=float(fields[2]),
                    dec=dec)

    tfdata = lines[8 + nch:]
    # Cycle through periods and extract data
    for i in range(nbt):
        # read period
        periods[0, i] = float(tfdata.pop(0).split()[2])

        # read number of data points
        ndf[i] = int(tfdata.pop(0).split()[4])

        # pop line
        tfdata.pop(0)

        # Cycle nche transfer functions
        for n in range(nche):
            tf = to_complex(tfdata.pop(0).split(), 2)
            z[0][i * nche + n] = tf[0]
            z[1][i * nche + n] = tf[1]

        # Pop line
        tfdata.pop(0)

        # Loop through Inverse Coherent Signal Power Matrix
        ncht = 2
        icspm = np.zeros((ncht, ncht), dtype=np.complex)
        for n in range(ncht):
            icspm[n] = to_complex(tfdata.pop(0).split(), ncht)
        icspm = fill_matrix(icspm)

        for n in range(ncht):
            sig_s[n][i * ncht:i * ncht + ncht] = icspm[n]

        # Pop line
        tfdata.pop(0)

        # Read residual covariance
        rc = np.zeros((nche, nche), dtype=np.complex)
        for n in range(nche):
            rc[n] = to_complex(tfdata.pop(0).split(), nche)

        rc = fill_matrix(rc)
        for n in range(len(rc)):
            sig_e[n][i * nche:i * nche + nche] = rc[n]

    # Assign to dictionary
    z_params['z'] = z
    z_params['sig_s'] = sig_s
    z_params['sig_e'] = sig_e
    z_params['periods'] = periods
    z_params['ndf'] = ndf
    z_params['stdec'] = stdec
    z_params['dec_format'] = dec_format
    z_params['orient'] = orient
    z_params['nch'] = nch
    z_params['nche'] = nche
    z_params['nbt'] = nbt
    z_params['chid'] = chid
    z_params['csta'] = csta
    z_params['station'] = station

    return z_params


def to_complex(lst: list, length: int) -> np.array:
    """Convert read string into complex numbers."""
    cmplx = np.zeros(length, dtype=np.complex)
    for n in range(int(len(lst) / 2)):
        if lst[2 * n + 1].startswith('-'):
            cmplx[n] = complex(f'{lst[2*n]}{lst[2*n+1]}j')
        else:
            cmplx[n] = complex(f'{lst[2*n]}+{lst[2*n+1]}j')
    return cmplx


def fill_matrix(mtrx: np.array) -> np.array:
    """Convert read information into Hermitian matrix."""
    for n in range(len(mtrx) - 1):
        mtrx[n][n + 1:] = np.transpose(np.conj(mtrx))[n][n + 1:]
    return mtrx


def ecomp(nch: int, chid: list) -> Tuple[list, list, list, list, str]:
    """Sort pairs of off diagonal Ex, Ey.

    Sort out and pair off diagonal Ex, Ey components for construction
    of conventional impedance tensors.

    Args:
        nch (int): Number of electrode channels.
        nchid (list): List of channel ids.

    Returns:
        xy (list): Index of Ex channels.
        yx (list): Index of Ey channels.
        hz (list): Index of Hz channels.
        xy_pairs (list): List of pairs xy, yx(?)
        dipole_setup (string): Type of array, "MT", "TEMAP", or "EMAP".

    """
    xy = []
    yx = []
    hz = []
    xy_pairs = []
    dipole_setup = ''

    nxy = 0
    nyx = 0

    for k in range(2, len(chid)):
        if chid[k].upper().startswith('EX'):
            xy.append(k - 2)
        elif chid[k].upper().startswith('EY'):
            yx.append(k - 2)
        elif chid[k].upper().startswith('H'):
            hz.append(k - 2)

    nxy = len(xy)
    nyx = len(yx)

    # Test three cases:
    # 1. All dipoles are in one line, throw error
    # 2. Conventional MT installation
    # 3. Multiple dipoles in one direction
    if min(nxy, nyx) == 0:
        dipole_setup = 'EMAP'
        print("Error: One or both of Ex/Ey not found in this file.")
    elif nxy == 1 and nyx == 1:
        dipole_setup = 'MT'
        xy_pairs = [xy, yx]
    else:
        dipole_setup = 'TEMAP'
        # Find "nearest" pairing of the available Ex/Ey components
        dist = abs((xy * np.ones((nyx, nxy), dtype=np.int64)) -
                   (np.transpose(yx) * np.ones((nyx, nyx), dtype=np.int64)))
        if nxy >= nyx:
            yxu = []
            mindist = np.amin(dist, axis=0)
            if nyx == 1:
                mindist = dist
            for k in range(nxy):
                for l in range(nyx):
                    if dist[l][k] == mindist[k]:
                        while(len(yxu) < k):
                            yxu.append(0)
                        yxu[k].append(yx[l])
            xy_pairs = xy + yxu
        else:
            xyu = []
            while(len(xyu) <= nyx):
                xyu.append(0)
            mindist = np.amin(dist, axis=1)
            if nxy == 1:
                mindist = dist
            for k in range(nyx):
                for l in range(nyx):
                    if dist[k][l] == mindist[k]:
                        xyu[k] = xy[l]
            xy_pairs = xyu + yx

    return (xy, yx, hz, xy_pairs, dipole_setup)


def z_to_tz(z: np.array, sig_s: np.array, sig_e: np.array, nper: int, rot: float) -> Tuple[np.array, np.array]:
    """Summary

    Args:
        z (np.array): Impedance matrix
        nper (int): Number of periods
        rot (float): Rotation angle

    Returns:
        Tuple(np.array, np.array): Description
    """
    theta = np.deg2rad(rot)
    R = np.array([[np.cos(theta), np.sin(theta)], [-np.sin(theta), np.cos(theta)]])
    R3 = np.array([[1, 0, 0], [0, R[0, 0], R[0, 1]], [0, R[1, 0], R[1, 1]]])
    S = np.zeros((nper, 4), dtype=np.complex)
    N = np.zeros((nper, 9), dtype=np.complex)
    tz = np.zeros((nper, 2), dtype=np.complex)
    tz_se = np.zeros((nper, 2), dtype=np.complex)

    for i, j in enumerate(range(0, sig_s.shape[1], 2)):
        tmp = np.matmul(R, sig_s[:, j:j + 2])
        S[i, :] = np.matmul(tmp, np.transpose(R)).reshape((1, 4))
        # S[i, :] = (R * sig_s[:, j:j + 2] * np.transpose(R)).reshape((1,4))
    for i, j in enumerate(range(0, z.shape[1] - 1, 3)):
        tz[i, :] = (np.matmul(R, z[:, j])).reshape((1, 2))
        tmp = np.matmul(R3, sig_e[:, j:j + 3])
        N[i, :] = np.matmul(tmp, np.transpose(R3)).reshape((1, 9))
        # N[i, :] = (R3 * sig_e[:, j:j + 3] * np.transpose(R3)).reshape((1,9))

    for i in range(nper):
        tz_se[i, 0] = np.sqrt(S[i, 0].real * N[i, 0])
        tz_se[i, 1] = np.sqrt(S[i, 3].real * N[i, 0])

    return tz, tz_se


def z_to_imp(z: np.array, sig_s: np.array, sig_e: np.array, nche: int,
             ixy: list, orient: list) -> Tuple[np.array, np.array, np.array]:
    """Translate input to one or more impedance matrices.

    The z_to_imp function translates input from general Z file to one or more
    impedance matrices with signal and noise covariance matrices
    necessary for full error computation in any coordinate system.

    Args:
        z (array): Complex transfer function.
        sig_s (array): Complex inverse signal covariance.
        sig_e (array): Complex residual error covariance.
        nche (int): Number of electrode channels.
        ixy (list): List of pairs xy, yx(?).
        orient (list): 2 x nche list of orientations and tilts.

    Returns:
        z2x2 (array): 4 x nbt impedance matrix, output order: Zxx, Zxy, Zyx, Zyy
        sigs (array): 4 x nbt signal covariance matrix.
        sige (array): 4 x nbt noise covariance matrix.

    """
    ntemp = np.shape(z)
    Nbt = int(ntemp[1] / nche)
    ntemp = np.shape(ixy)
    Nimp = ntemp[1]
    Nset = Nbt * Nimp
    orient = np.deg2rad(orient)

    # Make the rotaion matrix
    Th = np.array([[np.cos(orient[0][0]), np.cos(orient[0][1])],
                   [np.sin(orient[0][0]), np.sin(orient[0][1])]],
                  dtype=np.float64)
    Th = np.transpose(np.linalg.inv(Th))
    Uh = np.kron(Th, Th)

    z2x2 = np.zeros((4, Nset), dtype=np.complex)
    sige = np.zeros((4, Nset), dtype=np.complex)
    sigs = np.zeros((4, Nset), dtype=np.complex)

    for k in range(Nimp):
        a = k * Nset
        b = a + Nbt

        Te = np.array([[np.cos(orient[0][ixy[0][k] + 2]),
                        np.cos(orient[0][ixy[1][k] + 2])],
                       [np.sin(orient[0][ixy[0][k] + 2]),
                        np.sin(orient[0][ixy[1][k] + 2])]], dtype=np.float64)
        Ue = np.kron(Te, Te)
        Uz = np.kron(Te, Th)

        ztemp = np.vstack((z[:, ixy[0][k]::nche], z[:, ixy[1][k]::nche]))
        z2x2[:, a:b] = np.dot(Uz, ztemp)

        etemp = np.vstack((sig_e[ixy[0][k], ixy[0][k]::nche],
                           sig_e[ixy[0][k], ixy[1][k]::nche],
                           sig_e[ixy[1][k], ixy[0][k]::nche],
                           sig_e[ixy[1][k], ixy[1][k]::nche]))
        sige[:, a:b] = np.dot(Ue, etemp)

        stemp = np.vstack((sig_s[:, 0:2 * Nbt - 1:2], sig_s[:, 1:2 * Nbt:2]))
        sigs[:, a:b] = np.dot(Uh, stemp)

    return (z2x2, sigs, sige)


def rot_z(z2x2: np.array, sigs: np.array, sige: np.array, theta: float) \
        -> Tuple[np.array, np.array, np.array]:
    """Rotate array into theta.

    The rot_z function rotates the impedances, signal, and error covariance
    matrices into new coordinate system.

    Args:
        z2x2 (array): Impedance matrix, output order: Zxx, Zxy, Zyx, Zyy
        sigs (array): Signal covariance matrix
        sige (array): Noise covariance matrix
        theta (float): Rotation angle in degrees.

    Returns:
        z2x2r (array): Rotated impedance matix, output order:
                       Zxx, Zxy, Zyx, Zyy
        sigsr (array): Rotated signal covariance matrix
        siger (array): Rotate noise covariance matrix

    """
    c = np.cos(np.deg2rad(theta))
    s = np.sin(np.deg2rad(theta))

    U = np.array([[c * c, c * s, c * s, s * s],
                  [-c * s, c * c, -s * s, c * s],
                  [-c * s, -s * s, c * c, c * s],
                  [s * s, -c * s, -c * s, c * c]])

    z2x2r = np.dot(U, z2x2)
    sigsr = np.dot(U, sigs)
    siger = np.dot(U, sige)

    return (z2x2r, sigsr, siger)


def imp_ap(z: np.array, sigs: np.array, sige: np.array, periods: np.array, tz, tz_se) \
        -> Tuple[np.array, np.array, np.array, np.array]:
    """Compute apparent resistivity, phase, and errors.

    The imp_ap function computes the apparent resistivity, phase, and errors
    of the given impedance covariance matrices.  The output is in the final
    form of xy, yx pairs of rho and phi.

    Args:
        z (array): Impedance matrix, output order: Zxx, Zxy, Zyx, Zyy.
        sigs (array): Signal covariance matrix.
        sige (array): Noise covariance matrix.
        periods (array): Periods in
    Returns:
        rho (array): Impedance response: Rxx, Rxy, Ryx, Ryy.
        phi (array): Phase response.
        rho_se (array): Rho standard error.
        phi_se (array): Phi standard error.

    """
    data = namedtuple('data', 'rho rho_se phi phi_se tz tz_se')
    rad_deg = 180 / np.pi
    tmp = np.shape(periods)
    nbt = tmp[1]
    tmp = np.shape(z)
    nt = tmp[1]
    nimp = int(nt / nbt)
    ncmp = 4
    XX = 0
    XY = 1
    YX = 2
    YY = 3

    rho = np.zeros((nbt, ncmp * nimp), dtype=np.float64)
    phi = np.zeros((nbt, ncmp * nimp), dtype=np.float64)
    rho_se = np.zeros((nbt, ncmp * nimp), dtype=np.float64)
    phi_se = np.zeros((nbt, ncmp * nimp), dtype=np.float64)

    # Apparent resistivity
    rxx = np.zeros((nbt, nimp), dtype=np.float64)
    rxy = np.zeros((nbt, nimp), dtype=np.float64)
    ryx = np.zeros((nbt, nimp), dtype=np.float64)
    ryy = np.zeros((nbt, nimp), dtype=np.float64)
    # TODO: Need to properly calculate rxx_se, and ryy_se
    rxx_se = np.zeros((nbt, nimp), dtype=np.float64)
    rxy_se = np.zeros((nbt, nimp), dtype=np.float64)
    ryx_se = np.zeros((nbt, nimp), dtype=np.float64)
    ryy_se = np.zeros((nbt, nimp), dtype=np.float64)

    # Phase
    pxx = np.zeros((nbt, nimp), dtype=np.float64)
    pxy = np.zeros((nbt, nimp), dtype=np.float64)
    pyx = np.zeros((nbt, nimp), dtype=np.float64)
    pyy = np.zeros((nbt, nimp), dtype=np.float64)
    # TODO: Need to properly calculate pxx_se, and pyy_se
    pxx_se = np.zeros((nbt, nimp), dtype=np.float64)
    pxy_se = np.zeros((nbt, nimp), dtype=np.float64)
    pyx_se = np.zeros((nbt, nimp), dtype=np.float64)
    pyy_se = np.zeros((nbt, nimp), dtype=np.float64)

    for k in range(nimp):
        k1 = k * nbt
        k2 = k1 + nbt

        rxx[:, k] = np.square(np.abs(np.reshape(z[XX, k1:k2], (nbt))))
        rxy[:, k] = np.square(np.abs(np.reshape(z[XY, k1:k2], (nbt))))
        ryx[:, k] = np.square(np.abs(np.reshape(z[YX, k1:k2], (nbt))))
        ryy[:, k] = np.square(np.abs(np.reshape(z[YY, k1:k2], (nbt))))

        pxx[:, k] = np.dot(rad_deg, (np.arctan(
            np.reshape(np.imag(z[XX, k1:k2]), (nbt)) /
            np.reshape(np.real(z[XX, k1:k2]), (nbt)))))
        pxy[:, k] = np.dot(rad_deg, (np.arctan(
            np.reshape(np.imag(z[XY, k1:k2]), (nbt)) /
            np.reshape(np.real(z[XY, k1:k2]), (nbt)))))
        pyx[:, k] = np.dot(rad_deg, (np.arctan(
            np.reshape(np.imag(z[YX, k1:k2]), (nbt)) /
            np.reshape(np.real(z[YX, k1:k2]), (nbt)))))
        pyy[:, k] = np.dot(rad_deg, (np.arctan(
            np.reshape(np.imag(z[YY, k1:k2]), (nbt)) /
            np.reshape(np.real(z[YY, k1:k2]), (nbt)))))

        rxx_se[:, k] = np.real(np.reshape(sigs[XX, k1:k2], (nbt))) * \
            np.real(np.reshape(sige[XX, k1:k2], (nbt))) / 2
        rxy_se[:, k] = np.real(np.reshape(sigs[YY, k1:k2], (nbt))) * \
            np.real(np.reshape(sige[XX, k1:k2], (nbt))) / 2
        ryx_se[:, k] = np.real(np.reshape(sigs[XX, k1:k2], (nbt))) * \
            np.real(np.reshape(sige[YY, k1:k2], (nbt))) / 2
        ryy_se[:, k] = np.real(np.reshape(sigs[YY, k1:k2], (nbt))) * \
            np.real(np.reshape(sige[YY, k1:k2], (nbt))) / 2

    pxy_se = np.dot(rad_deg, np.sqrt(rxy_se / rxy))
    pyx_se = np.dot(rad_deg, np.sqrt(ryx_se / ryx))

    for l in range(nbt):
        rxx[l, :] = rxx[l, :] * periods[0, l] / 5
        rxy[l, :] = rxy[l, :] * periods[0, l] / 5
        ryx[l, :] = ryx[l, :] * periods[0, l] / 5
        ryy[l, :] = ryy[l, :] * periods[0, l] / 5

        rxx_se[l, :] = np.sqrt(rxx_se[l, :] * rxx[l, :] * periods[0, l] * 4 / 5)
        rxy_se[l, :] = np.sqrt(rxy_se[l, :] * rxy[l, :] * periods[0, l] * 4 / 5)
        ryx_se[l, :] = np.sqrt(ryx_se[l, :] * ryx[l, :] * periods[0, l] * 4 / 5)
        ryy_se[l, :] = np.sqrt(ryy_se[l, :] * ryy[l, :] * periods[0, l] * 4 / 5)

    for i in range(nimp):
        a = i * nimp
        b = a + ncmp

        rho[:, a:b] = np.stack((rxx[:, i], rxy[:, i], ryx[:, i], ryy[:, i]),
                               axis=-1)
        phi[:, a:b] = np.stack((pxx[:, i], pxy[:, i], pyx[:, i], pyy[:, i]),
                               axis=-1)
        rho_se[:, a:b] = 2 * np.stack((rxx_se[:, i], rxy_se[:, i],
                                       ryx_se[:, i], ryy_se[:, i]), axis=-1)
        phi_se[:, a:b] = 2 * np.stack((pxx_se[:, i], pxy_se[:, i],
                                       pyx_se[:, i], pyy_se[:, i]), axis=-1)

    return data(rho=rho, rho_se=rho_se, phi=phi, phi_se=phi_se, tz=tz, tz_se=tz_se)



from os.path import join

import matplotlib as mpl
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT
from matplotlib.backends.qt_compat import QtCore, QtWidgets


class NavigationToolbar(NavigationToolbar2QT):
    """Inherating from NavigationToolbar2QT."""

    # def _init_toolbar(self):
    def __init__(self, canvas, parent):
        """Override NavigationToolbar2QT _init_toolbar method."""
        super(NavigationToolbar, self).__init__(canvas, parent)

        # self.basedir = os.path.join(matplotlib.rcParams['datapath'], 'images')
        join(mpl.get_data_path(), 'images')

        # Clear the toolbar buttons, start with clean bar
        for x in self.actions():
            self.removeAction(x)

        # only display the buttons we want
        self.toolitems = [t for t in NavigationToolbar2QT.toolitems if
                          t[0] in ('Home', 'Pan', 'Zoom', 'Save', None)]

        for text, tooltip_text, image_file, callback in self.toolitems:
            if text is None:
                self.addSeparator()
            else:
                a = self.addAction(self._icon(image_file + '.png'),
                                   text, getattr(self, callback))
                self._actions[callback] = a
                if callback in ['zoom', 'pan']:
                    a.setCheckable(True)
                if tooltip_text is not None:
                    a.setToolTip(tooltip_text)
                if text == 'Zoom':
                    a = self.addAction(self._icon("subplots.png"),
                                       'Customize', self.edit_parameters)
                    a.setToolTip('Edit axis, curve and image parameters')

        # # self.buttons = {}

        # Add the x,y location widget at the right side of the toolbar
        # The stretch factor is 1 which means any resizing of the toolbar
        # will resize this label instead of the buttons.
        if self.coordinates:
            self.locLabel = QtWidgets.QLabel("", self)
            self.locLabel.setAlignment(
                QtCore.Qt.AlignRight | QtCore.Qt.AlignTop)
            self.locLabel.setSizePolicy(
                QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding,
                                      QtWidgets.QSizePolicy.Ignored))
            labelAction = self.addWidget(self.locLabel)
            labelAction.setVisible(False)

        # # reference holder for subplots_adjust window
        # # self.adj_window = None

        # # Esthetic adjustments - we need to set these explicitly in PyQt5
        # # otherwise the layout looks different - but we don't want to set it if
        # # not using HiDPI icons otherwise they look worse than before.
        self.setIconSize(QtCore.QSize(24, 24))
        self.layout().setSpacing(12)



#!/usr/bin/env python

"""The setup script."""

from setuptools import find_packages, setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['matplotlib>=3.4.1', 
                'PyQt5>=5.14.2', 
                'click>=7.1.2', 
                'numpy>=1.18.3']

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest>=3', ]

setup(
    author="Brady Fry",
    author_email='bradyfry@chaytus.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Tools for MT.",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='aprespy',
    name='aprespy',
    packages=find_packages(include=['aprespy', 'aprespy.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/chaytus/aprespy',
    version='2014.04.261',
    zip_safe=False,
)
